import binascii
from Crypto.PublicKey import RSA
from Crypto.Cipher import PKCS1_OAEP


def encrypt_rsa(data, pub_key_pem):
    public_key = RSA.import_key(pub_key_pem)
    cipher = PKCS1_OAEP.new(public_key)
    message_bytes = binascii.unhexlify(data)
    ciphertext = cipher.encrypt(message_bytes)
    ciphertext_hex = binascii.hexlify(ciphertext)
    return ciphertext_hex.decode()



import requests

from get_token import IRAN_KISH


def payment_verify(request):
    """
    This is a simple Django view which which is associated with the call_back_url.
    Just get the logic behind it!
    :param request:
    """
    if request.POST.get('resultCode') == '100':
        request_id = request.POST.get('InvoiceNumber')
        # do what you need to do before verification here

        payload = {
            'merchantId': request.POST.get('merchantId'),
            'referenceId': request.POST.get('referenceId'),
            'token': request.POST.get('token')
        }
        r = requests.post(IRAN_KISH['confirmation_url'], json=payload, verify=False)
        if r.ok:
            result = r.json()
            if result['status']:
                # transaction verification successful
                # do the necessary stuffs ...
                # redirect to a proper endpoint ...
                pass

            else:
                # transaction verification fail
                # do the necessary stuffs ...
                # redirect to a proper endpoint ...
                pass

    # transaction failed
    # do the necessary stuffs ...
    # redirect to a proper endpoint ...

    # you can also check the other result codes for more specific details. (read the ipg docs)



from iran_kish_payment.aes_encryptor import encrypt_aes
from iran_kish_payment.b_str_generator import generate_b_str
from iran_kish_payment.server_requester import request_to_server
from iran_kish_payment.rsa_encryptor import encrypt_rsa
from iran_kish_payment.sha256_hasher import sha256_hash


def request_payment(terminal_id: str, acceptor_id: str, password: str, amount: int, pub_key_pem: str, request_id: str,
                    revert_uri: str):
    b_str = generate_b_str(terminal_id=terminal_id, password=password, amount=amount)
    key, iv, ciphertext = encrypt_aes(b_str)
    hashed_data = sha256_hash(ciphertext)
    str_to_rsa = key + hashed_data
    encrypted_data = encrypt_rsa(data=str_to_rsa, pub_key_pem=pub_key_pem)

    res = request_to_server(terminal_id=terminal_id, acceptor_id=acceptor_id, amount=amount, iv=iv,
                            data=encrypted_data, request_id=request_id, revert_uri=revert_uri)
    # print(res)
    return res



from time import time
import requests


def request_to_server(iv: str, data: str, terminal_id: str, acceptor_id: str, amount: int, request_id: str,
                      revert_uri: str):
    timestamp = int(time())
    # pass the ssl error
    requests.packages.urllib3.disable_warnings()
    requests.packages.urllib3.util.ssl_.DEFAULT_CIPHERS += ':HIGH:!DH:!aNULL'
    try:
        requests.packages.urllib3.contrib.pyopenssl.util.ssl_.DEFAULT_CIPHERS += ':HIGH:!DH:!aNULL'
    except AttributeError:
        pass

    try:
        url = 'https://ikc.shaparak.ir/api/v3/tokenization/make'
        json_payload = {
            "authenticationEnvelope": {
                "iv": iv,
                "data": data
            },
            "request": {
                "transactionType": "Purchase",
                "terminalId": terminal_id,
                "acceptorId": acceptor_id,
                "amount": int(amount),
                "revertUri": revert_uri,
                "requestId": request_id,
                "requestTimestamp": timestamp,
            }
        }
        headers = {
            'Content-Type': 'application/json',
        }
        response = requests.post(url=url, json=json_payload, headers=headers, verify=False)
        response.raise_for_status()
    except requests.exceptions.ConnectionError as e:
        print('ConnectionError: ', e)
    except requests.exceptions.RequestException as e:
        print('RequestException: ', e)
    except Exception as e:
        print('Exception: ', e)
    else:
        try:
            return response.json()
        except ValueError:
            print('Response content is not in JSON format:', response.text)



import pathlib

from setuptools import setup, find_packages

HERE = pathlib.Path(__file__).parent
README = (HERE / "README.md").read_text()

setup(
    name="iran_kish_py",
    version="1.0.0",
    packages=find_packages(),
    install_requires=[
        'pycryptodome',
        'requests',
        'urllib3<2'
    ],
    tests_require=[
        'unittest',
    ],
    test_suite='tests',
    description="Iran Kish payment portal python package",
    long_description=README,
    long_description_content_type="text/markdown",
    author="fastpanda99",
    author_email="fastpanda99@gmail.com",
    url="https://github.com/fastpanda99/iran_kish_py",
)



from Crypto.Cipher import AES
from Crypto.Util.Padding import pad, unpad
from Crypto.Random import get_random_bytes
import hashlib


def encrypt_aes(data: str):
    binary_data = bytes.fromhex(data)
    key = get_random_bytes(16)  # 128-bit key
    iv = get_random_bytes(16)
    padded_data = pad(binary_data, AES.block_size)
    cipher = AES.new(key, AES.MODE_CBC, iv)
    ciphertext = cipher.encrypt(padded_data)
    return key.hex(), iv.hex(), ciphertext
